#include "tal_memory.h"

#include "app_common.h"
#include "app_key.h"
#include "app_mfg.h"
#include "bsp.h"

UINT8_T                 net_count = 0;
UINT8_T                 led_status = 0;
UINT8_T                 general_control = 0;
UINT8_T                 tuya_gpio_key_value_bak = 0;

receiver_status_t       device_status;
user_flash_store_t      user_flash_store;
#if 0
TIMER_ID                sleep_timer_id;
TIMER_ID                allow_sleep_timer_id;
#endif
TIMER_ID                ext_key_allow_timer_id;
// TIMER_ID                update_timer_id;
TIMER_ID                flash_update_timer_id;
TIMER_ID                flash_timer_id;
TIMER_ID                zigbee_update_timer_id;

/********************************************************************************
 * @brief Writes the user flash data structure to non-volatile memory.
 * @author ysl
 * @return void
 * @date 2025.07
 *
 * This function performs the following steps:
 * - Resets all NV storage (calls tal_nv_reset_all).
 * - Writes the user_flash_store data to flash using tal_nv_flash_write.
 * @note Function: _fun_flash_write
 ********************************************************************************/
STATIC VOID_T _fun_flash_write(VOID_T)
{
  TAL_NV_STATUS_T v_ret = TAL_NV_SUCC;
      TAL_PR_TRACE("flash write");
  v_ret = tal_nv_reset_all();
  if (v_ret != TAL_NV_SUCC)
  {
    TAL_PR_DEBUG("_fun_flash_write  tal_nv_reset_all.");
    return;
  }
  v_ret = tal_nv_flash_write(KV_ID_APP_DATA, flash_timer_id, sizeof(user_flash_store), (UCHAR_T *)&user_flash_store);
  if (v_ret != TAL_NV_SUCC)
  {
    TAL_PR_DEBUG("_fun_flash_write  clear reset cnt failed.");
    return;
  }
}

/********************************************************************************
 * @brief Reads the user flash data structure from non-volatile memory.
 * @author ysl
 * @return void
 * @date 2025.07
 *
 * This function attempts to restore user_flash_store from flash.
 * If the read fails, an error message is printed.
 * @note Function: _fun_flash_read
 ********************************************************************************/
STATIC VOID_T _fun_flash_read(VOID_T)
{
  TAL_NV_STATUS_T v_ret = TAL_NV_SUCC;
  v_ret = tal_nv_flash_read(KV_ID_APP_DATA, flash_timer_id, sizeof(user_flash_store), (UCHAR_T *)&user_flash_store);
  if (v_ret != TAL_NV_SUCC)
  {
    TAL_PR_DEBUG("_fun_flash_read  clear reset cnt failed.");
    return;
  }
}

/********************************************************************************
 * @brief Timer callback to update relay status into flash.
 * @author ysl
 * @return void
 * @date 2025.07
 *
 * This callback saves the current relay status into user_flash_store
 * and writes it to non-volatile memory.
 * @note Function: flash_update_timer_cb
 ********************************************************************************/
VOID_T flash_update_timer_cb(TIMER_ID timer_id, VOID_T *arg)
{
  user_flash_store.state_config.on = device_status.relStatus;
    TAL_PR_TRACE("flash_update_timer_cb device_status.relStatus %d", device_status.relStatus);
  _fun_flash_write();
}

/********************************************************************************
 * @brief General-purpose flash timer callback for logging or extension.
 * @author ysl
 * @return void
 * @date 2025.07
 *
 * Currently prints a log message. Can be used for delayed flash operations.
 * @note Function: __fun_flash_timer_cb
 ********************************************************************************/
STATIC VOID_T __fun_flash_timer_cb(TIMER_ID timer_id, VOID_T *arg)
{
  TAL_PR_TRACE("__fun_flash_timer_cb");
}

/********************************************************************************
 * @brief Starts a timer to trigger delayed flash update.
 * @author ysl
 * @return void
 * @date 2025.07
 *
 * Called when the relay state changes to defer flash writes.
 * @note Function: _fun_time_update
 ********************************************************************************/
STATIC VOID_T _fun_time_update(VOID_T)
{
  TAL_PR_NOTICE("_fun_time_update: ");
  // __fun_update_timer_cb();
  tal_sw_timer_start(flash_update_timer_id, SLEEP_TIMER_INTERVAL_MS, TAL_TIMER_ONCE);
  // tal_sw_timer_start(update_timer_id, FLASH_Tx1 IMER_INTERVAL_MS, TAL_TIMER_ONCE);
}

/********************************************************************************
 * @brief Retrieves the status of the specified relay channel.
 * @author ysl
 * @param[in] channel - The relay channel index to check.
 * @return 0 if the channel is off, non-zero if it is on.
 * @date 2025.07
 *
 * This function checks whether the specified bit is set in the relStatus field
 * of the global device_status structure.
 * @note Function: get_rel_status
 ********************************************************************************/
UINT8_T get_rel_status(UINT8_T channel)
{
  return device_status.relStatus & (1 << channel);
}

/********************************************************************************
 * @brief Turns on the specified relay channel and optionally reports the change.
 * @author ysl
 * @param[in] channel - Relay channel to turn on
 * @param[in] is_update - Whether to send a report (REPORT_ENABLE)
 * @return void
 * @date 2025.07
 *
 * This function:
 * - Sets the corresponding bit in device_status.relStatus
 * - Calls _fun_time_update to defer flash write
 * - Optionally triggers on/off report
 * @note Function: rel_on
 ********************************************************************************/
VOID_T rel_on(UINT8_T channel, report_ctrl_t is_update)
{
  if ((device_status.relStatus & (1 << channel))) // || networkIsConnected)
    return;
  TAL_PR_TRACE("rel_on");
  device_status.relStatus |= (1 << channel);
  set_rel_status(channel);
  // set_led_status(channel, LED_ON);
  // set_release_fun(channel);
  _fun_time_update();
  if (is_update == REPORT_ENABLE)
    app_onoff_cluster_write_report(channel + 1, LED_ON);
}

/********************************************************************************
 * @brief Turns off the specified relay channel and optionally reports the change.
 * @author ysl
 * @param[in] channel - Relay channel to turn off
 * @param[in] is_update - Whether to send a report (REPORT_ENABLE)
 * @return void
 * @date 2025.07
 *
 * This function:
 * - Clears the corresponding bit in device_status.relStatus
 * - Calls _fun_time_update to defer flash write
 * - Optionally triggers on/off report
 * @note Function: rel_off
 ********************************************************************************/
VOID_T rel_off(UINT8_T channel, report_ctrl_t is_update)
{
  if (!(device_status.relStatus & (1 << channel)))
    return;
  TAL_PR_TRACE("rel_off");
  device_status.relStatus &= ~(1 << channel);
  set_rel_status(channel);
  // set_led_status(channel, LED_OFF);

  // upload_device_switch_status(channel);
  _fun_time_update();
  if (is_update == REPORT_ENABLE)
    app_onoff_cluster_write_report(channel + 1, LED_OFF);
}

#if 0
/**
 * @brief ˯߶ʱصÿ 2 һ
 * 1. ǿƻ
 * 2. һӳٶʱ˯
 */
VOID_T sleep_timer_cb(TIMER_ID timer_id, VOID_T *arg)
{
  // tal_cpu_force_wakeup();
  tal_cpu_force_wakeup();
  // TAL_PR_TRACE("sleep_timer_cb : %d", SLEEP_TIMER_INTERVAL_MS);

  // ӳ 100ms ˯
  tal_sw_timer_start(allow_sleep_timer_id, SLEEP_ALLOW_DELAY_MS, TAL_TIMER_ONCE);

  // ظʱ2ٴλ
  tal_sw_timer_start(sleep_timer_id, SLEEP_TIMER_INTERVAL_MS, TAL_TIMER_ONCE);
}

/**
 * @brief ˯ߵĻص
 */
VOID_T sleep_allow_cb(TIMER_ID timer_id, VOID_T *arg)
{
  TAL_PR_TRACE(">>> allow sleep now <<<");
  tal_cpu_allow_sleep();
}
#endif

/********************************************************************************
 * @brief Timer callback for detecting long press on external wired key.
 * @author ysl
 * @param[in] timer_id - Timer ID
 * @param[in] arg - Timer argument (unused)
 * @return void
 * @date 2025.07
 *
 * This function:
 * - Decrements wired_key_count
 * - Updates switch type if long press is confirmed
 * @note Function: ext_key_allow_cb
 ********************************************************************************/
VOID_T ext_key_allow_cb(TIMER_ID timer_id, VOID_T *arg) // 100ms
{
  device_status.wired_key_count--;
  if (device_status.wired_key_count == LONG_PRESS_END)
  {
    device_status.ucSwitchType[device_status.wired_key_num] = 1;
    device_status.wired_key_count = LONG_PRESS_TIME_CONST;
    tal_sw_timer_stop(ext_key_allow_timer_id);
  }
}

/********************************************************************************
 * @brief Periodic timer callback to send Zigbee attribute reports.
 * @author ysl
 * @param[in] timer_id - Timer ID
 * @param[in] arg - Timer argument (unused)
 * @return void
 * @date 2025.07
 *
 * This callback calls all_update_data to sync relay states.
 * @note Function: zigbee_update_timer_cb
 ********************************************************************************/
VOID_T zigbee_update_timer_cb(TIMER_ID timer_id, VOID_T *arg) // 100ms
{
  all_update_data();
}

/********************************************************************************
 * @brief Initializes software timers used for key and flash operations.
 * @author ysl
 * @return void
 * @date 2025.07
 *
 * Creates all application-level timers including:
 * - ext_key_allow_timer_id
 * - flash_update_timer_id
 * - flash_timer_id
 * - zigbee_update_timer_id
 * @note Function: user_timer_init
 ********************************************************************************/
STATIC VOID_T user_timer_init(VOID_T)
{
#if 0
  // ʼʱʱ
  tal_sw_timer_create(sleep_timer_cb, NULL, &sleep_timer_id);
  tal_sw_timer_create(sleep_allow_cb, NULL, &allow_sleep_timer_id);
#endif
  tal_sw_timer_create(ext_key_allow_cb, NULL, &ext_key_allow_timer_id);
  tal_sw_timer_create(flash_update_timer_cb, NULL, &flash_update_timer_id);
  tal_sw_timer_create(__fun_flash_timer_cb, NULL, &flash_timer_id);
  tal_sw_timer_create(zigbee_update_timer_cb, NULL, &zigbee_update_timer_id);
  // tal_sw_timer_start(sleep_timer_id, SLEEP_TIMER_INTERVAL_MS, TAL_TIMER_ONCE);
}

/********************************************************************************
 * @brief Restores relay states from flash and applies them at startup.
 * @author ysl
 * @param[in] user_flash_p - Pointer to user flash structure
 * @return void
 * @date 2025.07
 *
 * This function:
 * - Reads flash data
 * - Turns on relays whose bits are set in the saved state
 * @note Function: user_data_init
 ********************************************************************************/
STATIC VOID_T user_data_init(user_flash_store_t *user_flash_p)
{
  UINT8_T i;
  UINT8_T read_on;

  _fun_flash_read();
  TAL_PR_TRACE("user_data_init   user_flash_store.state_config.on %d", user_flash_store.state_config.on);
  read_on = user_flash_store.state_config.on;
  for (i = 0; i < CHANNEL_NUM; i++)
  {
    TAL_PR_TRACE("user_data_init   read_on %d", read_on);
    if (read_on & (1 << i))
    {
      TAL_PR_TRACE("user_data_init   rel_on %d", i);
      rel_on(i, REPORT_ENABLE);
    }
  }

}

/********************************************************************************
 * @brief Initializes all device parameters and state during power-up.
 * @author ysl
 * @return void
 * @date 2025.07
 *
 * This function:
 * - Clears device and flash state
 * - Initializes timers and relay state from flash
 * - Reads external key status
 * @note Function: device_init
 ********************************************************************************/
VOID_T device_init(VOID_T)
{
  UINT8_T i;
  TUYA_GPIO_NUM_E key_pins[CHANNEL_NUM] = {EXT_KEY_PIN_1, EXT_KEY_PIN_2, EXT_KEY_PIN_3, EXT_KEY_PIN_4};
  memset(&device_status, 0, sizeof(device_status));
  memset(&user_flash_store, 0, sizeof(user_flash_store));

  user_timer_init();

  user_data_init(&user_flash_store);

  tuya_gpio_key_value_bak = gpio_ext_key_read_keys(key_pins);
  for (i = 0; i < CHANNEL_NUM; i++)
  {
    device_status.ucSwitchType[i] = 0;
  }
  device_status.wired_key_count = LONG_PRESS_TIME_CONST;
}

