/*
 * @Author: zhangshu
 * @email: shiliang.zhang@tuya.com
 * @LastEditors: Please set LastEditors
 * @file name: app_led.c
 * @Description:
 * @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
 * @Company: http://www.tuya.com
 * @Date: 2022-02-08
 * @LastEditTime: 2022-02-08
 */
// #include "tuya_hal_gpio.h"

#include "tal_memory.h"
#include "tal_sw_timer.h"
#include "tal_gpio.h"
#include "tal_network_mgr.h"
#include "tal_mf_test.h"
#include "tal_sleep.h"

#include "app_common.h"
#include "app_led.h"
#include "app_gpio.h"
#include "app_ias_zone.h"
#include "bsp.h"

// led blink type number
#define LED_BLINK_TYPE_NUM 9
// blink forver times
#define LED_BLINK_FOREVER 0xFFFFFFFF
// blink forever count
#define LED_BLINK_FOREVER_COUNT 0xFFFF

// led state
typedef enum
{
  LED_STATE_NONE,
  LED_STATE_ON,
  LED_STATE_OFF,
} LED_STATE_E;

// led blink type
typedef enum
{
  LED_BLINK_TYPE_NONE,         // blink none, default type
  LED_BLINK_TYPE_POWER_ON,     // blink when power on
  LED_BLINK_TYPE_START_JOIN,   // blink when start join network
  LED_BLINK_TYPE_JOIN_SUCCESS, // blink when join success
  LED_BLINK_TYPE_BUTTON_DOWN,  // solid when button down
  LED_BLINK_TYEP_ALARM,        // blink when door open
  LED_BLINK_TYEP_TAMPER,       // blink when tamper trigger
  LED_BLINK_TYPE_IDENTIFY,     // blink when identify
  LED_BLINK_TYPE_MF_TEST,
} LED_BLINK_TYPE_E;

// blink parameter
typedef struct
{
  LED_BLINK_TYPE_E type; // blink type
  UINT8_T start_off;     // start with off or not
  UINT16_T count;        // blink count
  UINT16_T on_ms;        // on time duration in ms
  UINT16_T off_ms;       // off time duration in ms
} BLINK_PARAMETER_T;

// blink array include type and priority
typedef struct
{
  LED_BLINK_TYPE_E type; // blink type
  UINT8_T priority;      // blink priority
} BLINK_ARRAY_T;

// blink type and corresponding piority
STATIC CONST BLINK_ARRAY_T blink_array[LED_BLINK_TYPE_NUM] = {
    {LED_BLINK_TYPE_NONE, 0xFF},
    {LED_BLINK_TYPE_POWER_ON, 1},
    {LED_BLINK_TYPE_START_JOIN, 3},
    {LED_BLINK_TYPE_JOIN_SUCCESS, 2},
    {LED_BLINK_TYPE_BUTTON_DOWN, 4},
    {LED_BLINK_TYEP_ALARM, 6},
    {LED_BLINK_TYEP_TAMPER, 7},
    {LED_BLINK_TYPE_IDENTIFY, 5},
    {LED_BLINK_TYPE_MF_TEST, 0},
};

// blink para
STATIC BLINK_PARAMETER_T blink_parameter = {LED_BLINK_TYPE_NONE, 0, 0, 0, 0};
// led state
STATIC LED_STATE_E led_state = LED_STATE_NONE;
// led info
STATIC APP_LED_INFO_T *led_info;

// led blink event and handler
TIMER_ID etimer_led_blink_event_id;
STATIC VOID_T app_sensor_led_blink_event_handler(TIMER_ID timer_id, VOID_T *arg);

// led event create
STATIC VOID_T app_led_event_init(VOID_T);
// led blink complete callback
STATIC VOID_T led_blink_complete_callback(LED_BLINK_TYPE_E type);
// led stop all led
STATIC VOID_T led_stop(VOID_T);
// set status led gpio status
STATIC VOID_T status_led_set(BOOL_T onoff);
// set network led gpio status
STATIC VOID_T network_led_set(BOOL_T onoff);

extern BOOL_T mf_test_doing_flag;
extern BOOL_T sys_init_finish_flag;

OPERATE_RET app_led_oem_init()
{

  TUYA_GPIO_BASE_CFG_T status_config;

  led_info = (APP_LED_INFO_T *)tal_malloc(SIZEOF(APP_LED_INFO_T));

  tal_system_memset(led_info, 0, SIZEOF(APP_LED_INFO_T));
  status_config.mode = TUYA_GPIO_PUSH_PULL;
  status_config.direct = TUYA_GPIO_OUTPUT;
  status_config.level = TUYA_GPIO_LEVEL_LOW;

  tal_gpio_init(LED_PIN_1, &status_config);

  status_config.level  = TUYA_GPIO_LEVEL_LOW;        // ʼ͵ƽҪߵƽΪ HIGH
  tal_gpio_init(REL_PIN_1, &status_config);
  tal_gpio_init(REL_PIN_2, &status_config);
  tal_gpio_init(REL_PIN_3, &status_config);
  tal_gpio_init(REL_PIN_4, &status_config);
}


/**
 * @description: led init after preheat time delay, gpio state may not stable during the preheat time
 * @param[in] none
 * @return: none
 */
VOID_T app_led_preheat_init(VOID_T)
{
  app_led_event_init();
  dev_power_on_indicate();
}

/**
 * @description: led blink event create
 * @param[in] none
 * @return: none
 */
STATIC VOID_T app_led_event_init(VOID_T)
{
  tal_sw_timer_create(app_sensor_led_blink_event_handler, NULL, &etimer_led_blink_event_id);
}

/**
 * @description: set_blink_event
 * @param[in] type: blink type, refer to LED_BLINK_TYPE_E
 * @param[in] on_ms: on time of duration in ms
 * @param[in] off_ms: off time of duration in ms
 * @param[in] delay_ms: delay start blink
 * @param[in] start_off: start with off state or not
 * @return: none
 */
VOID_T set_blink_event(LED_BLINK_TYPE_E type, UINT16_T on_ms, UINT16_T off_ms, UINT32_T dur_ms, UINT16_T delay_ms, UINT8_T start_off)
{
  if (blink_array[type].priority > blink_array[blink_parameter.type].priority)
  {
    TAL_PR_TRACE("blink priority low, do not set blink\r\n");
    return;
  }

  led_state = LED_STATE_ON;
  tal_system_memset(&blink_parameter, 0, SIZEOF(BLINK_PARAMETER_T));
  led_stop();

  blink_parameter.type = type;
  blink_parameter.on_ms = on_ms;
  blink_parameter.off_ms = off_ms;
  blink_parameter.start_off = start_off;

  if (dur_ms == LED_BLINK_FOREVER)
  {
    blink_parameter.count = LED_BLINK_FOREVER_COUNT;
  }
  else
  {
    blink_parameter.count = (UINT16_T)(dur_ms / (on_ms + off_ms));
  }

  tal_sw_timer_start(etimer_led_blink_event_id, delay_ms, TAL_TIMER_ONCE);
  TAL_PR_TRACE("SET BLINK, on time %d, off time %d, count %d\r\n", blink_parameter.on_ms, blink_parameter.off_ms, blink_parameter.count);
}

/**
 * @description: led blink event
 * @param [in] timer_id: timer id
 * @param[in] args: null
 * @return: none
 */
STATIC VOID_T app_sensor_led_blink_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
  // if (led_status & (1 << 0))
  // {
  //   set_led_status(0, LED_ON);
  //   tal_cpu_force_wakeup();
  //   tal_sw_timer_start(etimer_led_blink_event_id, 250, TAL_TIMER_ONCE);
  //   return;
  // }
  // else if (!(led_status & (1 << 0)))
  // {
  //     net_count--;
  //     if (blink_parameter.off_ms != LED_BLINK_FOREVER_COUNT) {
  //         tal_sw_timer_start(etimer_led_blink_event_id, blink_parameter.off_ms, TAL_TIMER_ONCE);
  //     }
  //     if (blink_parameter.count) {
  //         set_led_status(0, LED_OFF);
  //         return;
  //     }
  // }
  // else{
  //   set_led_status(0, LED_OFF);
  // }
  // TAL_PR_TRACE("device_status.ledStatus   %d---\r\n", device_status.ledStatus);
  // TAL_PR_TRACE("LED_STATUS_ZIGBEE_RESET%d,  LED_STATUS_PRESS_SLOW  %d---\r\n", device_status.led & LED_STATUS_ZIGBEE_RESET, device_status.led & LED_STATUS_PRESS_SLOW);
  if (device_status.led & (LED_STATUS_ZIGBEE_RESET | LED_STATUS_PRESS_SLOW))
  {
    UINT32_T blink_time = 300;
    if (device_status.led & LED_STATUS_PRESS_SLOW)
    {
      blink_time = 1000;
    }
    if (device_status.ledStatus)
    {
      set_led_status(0, LED_OFF);
    }
    else
    {
      set_led_status(0, LED_ON);
    }
    tal_sw_timer_start(etimer_led_blink_event_id, blink_time, TAL_TIMER_ONCE);
  }
  // if (device_status.ledStatus == true)
  // {
  //   if (device_status.led & LED_STATUS_ZIGBEE_RESET)
  //   {
  //     TAL_PR_TRACE("LED_STATUS_ZIGBEE_RESET  LED_ON   ---\r\n");
  //     set_led_status(0, LED_OFF);
  //     tal_sw_timer_start(etimer_led_blink_event_id, 500, TAL_TIMER_ONCE);
  //   }
  // }
  // else if (device_status.ledStatus == false)
  // {
  //   if (device_status.led & LED_STATUS_ZIGBEE_RESET)
  //   {
  //     TAL_PR_TRACE("LED_STATUS_ZIGBEE_RESET  LED_OFF   ---\r\n");
  //     set_led_status(0, LED_ON);

  //     tal_sw_timer_start(etimer_led_blink_event_id, 500, TAL_TIMER_ONCE);
  //   }
  // }
}

/**
 * @description: led blink complete callback
 * @param[in] type: blink type, refer to LED_BLINK_TYPE_E
 * @return: none
 */
STATIC VOID_T led_blink_complete_callback(LED_BLINK_TYPE_E type)
{
  switch (type)
  {

  case LED_BLINK_TYPE_POWER_ON:
  case LED_BLINK_TYPE_JOIN_SUCCESS:
  {
    if (tal_zg_nwk_status_get() == TAL_ZG_NWK_JOIN_START)
    {
      dev_start_join_indicate();
    }
    else
    {
      if (is_in_alarm_state() && (LED_INDICATE_TYPE_SOLID == led_info->alarm_type))
      {
        dev_alarm_indicate(TRUE);
      }
    }
    break;
  }
  case LED_BLINK_TYPE_BUTTON_DOWN:
  case LED_BLINK_TYPE_IDENTIFY:
  {
    if (is_in_alarm_state() && (LED_INDICATE_TYPE_SOLID == led_info->alarm_type))
    {
      dev_alarm_indicate(TRUE);
    }
    break;
  }
  case LED_BLINK_TYPE_START_JOIN:
  {
    if (is_in_alarm_state() && (LED_INDICATE_TYPE_SOLID == led_info->alarm_type))
    {
      dev_alarm_indicate(TRUE);
    }
    TAL_PR_TRACE("join led timeout\r\n");
    break;
  }
  case LED_BLINK_TYEP_ALARM:
  {
    TAL_PR_TRACE("alarm indicate complete\r\n");
    break;
  }
  case LED_BLINK_TYEP_TAMPER:
  {
    TAL_PR_TRACE("tamper indicate complete\r\n");
    break;
  }
  default:
    break;
  }
}

/**
 * @description: stop led blink
 * @param[in] none
 * @return: none
 */
VOID_T stop_blink(VOID_T)
{
  tal_sw_timer_stop(etimer_led_blink_event_id);
  led_state = LED_STATE_NONE;
  tal_system_memset(&blink_parameter, 0, SIZEOF(BLINK_PARAMETER_T));
  blink_parameter.type = LED_BLINK_TYPE_NONE;
  led_stop();
}

/**
 * @description: stop all led
 * @param[in] none
 * @return: none
 */
STATIC VOID_T led_stop(VOID_T)
{
  TAL_PR_TRACE("led_stop \r\n");
  status_led_set(FALSE);
  network_led_set(FALSE);
  if ((!mf_test_doing_flag) || (sys_init_finish_flag))
  {
    TAL_PR_TRACE("tal_cpu_allow_sleep \r\n");
    tal_cpu_allow_sleep();
  }
}

/**
 * @description: power on solid indicate
 * @param[in] none
 * @return: none
 */
VOID_T dev_power_on_indicate(VOID_T)
{
  set_blink_event(LED_BLINK_TYPE_POWER_ON, 3000, 200, 3200, 0, 0);
}

/**
 * @description: device start join led indicate
 * @param[in] none
 * @return: none
 */
VOID_T dev_start_join_indicate(VOID_T)
{
  set_blink_event(LED_BLINK_TYPE_START_JOIN, 250, 250, ZIGBEE_JOIN_TIMEOUT_MS, 0, 0);
}

/**
 * @description: device join network success led indicate
 * @param[in] none
 * @return: none
 */
VOID_T dev_join_success_indicate(VOID_T)
{
  set_blink_event(LED_BLINK_TYPE_JOIN_SUCCESS, 3000, 200, 3200, 0, 0);
}

/**
 * @description: device alarm indicate
 * @param[in] alarm: TRUE when alarm or not
 * @return: none
 */
VOID_T dev_alarm_indicate(BOOL_T alarm)
{
  if (alarm)
  {
    if (LED_INDICATE_TYPE_NONE == led_info->alarm_type)
    {
      return;
    }
    else if (LED_INDICATE_TYPE_ONCE == led_info->alarm_type)
    {
      set_blink_event(LED_BLINK_TYEP_ALARM, 300, 0, 300, 0, 0);
    }
    else if (LED_INDICATE_TYPE_SOLID == led_info->alarm_type)
    {
      set_blink_event(LED_BLINK_TYEP_ALARM, LED_BLINK_FOREVER_COUNT, 0, LED_BLINK_FOREVER, 0, 0);
    }
  }
  else
  {
    if (blink_array[LED_BLINK_TYEP_ALARM].priority == blink_array[blink_parameter.type].priority)
    {
      stop_blink();
      led_blink_complete_callback(LED_BLINK_TYEP_ALARM);
    }
  }
}

/**
 * @description: device tamper indicate
 * @param[in] tamper: TRUE when tamper or not
 * @return: none
 */
VOID_T dev_tamper_indicate(BOOL_T tamper)
{
  if (tamper)
  {
    if (LED_INDICATE_TYPE_NONE == led_info->tamper_type)
    {
      return;
    }
    else if (LED_INDICATE_TYPE_ONCE == led_info->tamper_type)
    {
      set_blink_event(LED_BLINK_TYEP_TAMPER, 300, 0, 300, 0, 0);
    }
    else if (LED_INDICATE_TYPE_SOLID == led_info->tamper_type)
    {
      set_blink_event(LED_BLINK_TYEP_TAMPER, LED_BLINK_FOREVER_COUNT, 0, LED_BLINK_FOREVER, 0, 0);
    }
  }
  else
  {
    if (blink_array[LED_BLINK_TYEP_TAMPER].priority == blink_array[blink_parameter.type].priority)
    {
      stop_blink();
      led_blink_complete_callback(LED_BLINK_TYEP_TAMPER);
    }
  }
}

VOID_T set_led_blink_event(VOID_T)
{
  net_count = (UINT16_T)(60 * 1000 / (500));
  tal_sw_timer_start(etimer_led_blink_event_id, 0, TAL_TIMER_ONCE);
}

VOID_T set_led_status(UINT8_T channel, UINT8_T led_status)
{
  TUYA_GPIO_NUM_E led_pins[1] = {LED_PIN_1};
  if (led_status)
    device_status.ledStatus = true;
  else
    device_status.ledStatus = false;
  if (channel < CHANNEL_NUM)
  {
    TUYA_GPIO_NUM_E pin = led_pins[channel];
    if (led_status == LED_ON)
    {
      // tal_cpu_force_wakeup();
      tkl_gpio_write(pin, TUYA_GPIO_LEVEL_HIGH);
      // tal_cpu_allow_sleep();
    }
    else if (led_status == LED_OFF)
    {
      // tal_cpu_force_wakeup();
      tkl_gpio_write(pin, TUYA_GPIO_LEVEL_LOW);
      // tal_cpu_allow_sleep();
    }
  }
  else if (channel == LED_STATUS_ZIGBEE_RESET)
  {
  }
}

/**
 * @description: button press indicate
 * @param[in] none
 * @return: none
 */
VOID_T button_press_on_indicate(VOID_T)
{
  set_blink_event(LED_BLINK_TYPE_BUTTON_DOWN, LED_BLINK_FOREVER_COUNT, 0, LED_BLINK_FOREVER, 0, 0);
  TAL_PR_TRACE("led_state %d \r\n", led_state);
}

/**
 * @description: button release indicate
 * @param[in] none
 * @return: none
 */
VOID_T button_release_off_indicate(VOID_T)
{
  if (blink_array[LED_BLINK_TYPE_BUTTON_DOWN].priority == blink_array[blink_parameter.type].priority)
  {
    stop_blink();
    led_blink_complete_callback(LED_BLINK_TYPE_BUTTON_DOWN);
  }
}

/**
 * @description: identify indicate
 * @param[in] time_ms: identify time in ms
 * @return: none
 */
VOID_T identify_indicate(UINT32_T time_ms)
{
  if (time_ms == LED_BLINK_FOREVER)
  {
    set_blink_event(LED_BLINK_TYPE_IDENTIFY, 500, 500, LED_BLINK_FOREVER, 0, 1);
  }
  else
  {
    set_blink_event(LED_BLINK_TYPE_IDENTIFY, 500, 500, time_ms, 0, 1);
  }
}

/**
 * @description: manufacture test blink
 * @param[in] cmd: manufacture command, refer to TUYA_MF_TEST_CMD_T
 * @return: none
 */
VOID_T mf_test_blink(TUYA_MF_TEST_CMD_T cmd)
{
  switch (cmd)
  {
  case TUYA_MF_TEST_LED_ON_ALL:
  {
    blink_parameter.type = LED_BLINK_TYPE_MF_TEST;
    network_led_set(TRUE);
    status_led_set(TRUE);
    if ((!mf_test_doing_flag) || (sys_init_finish_flag))
    {
      tal_cpu_force_wakeup();
    }
    break;
  }
  case TUYA_MF_TEST_LED_OFF_ALL:
  {
    stop_blink();
    break;
  }
  case TUYA_MF_TEST_LED_BLINK_ALL:
  {
    set_blink_event(LED_BLINK_TYPE_MF_TEST, 500, 500, LED_BLINK_FOREVER, 0, 1);
    break;
  }
  default:
  {
    break;
  }
  }
}

/**
 * @description: set alarm led on off state
 * @param[in] state: on off state
 * @return: none
 */
VOID_T set_alarm_led_on_off(UINT8_T state)
{
  if (state)
  {
    if (is_in_alarm_state() && (LED_INDICATE_TYPE_SOLID == led_info->alarm_type))
    {
      dev_alarm_indicate(TRUE);
    }
  }
  else
  {
    if (is_in_alarm_state() && (LED_INDICATE_TYPE_SOLID == led_info->alarm_type) && (blink_parameter.type == LED_BLINK_TYEP_ALARM))
    {
      stop_blink();
    }
  }
}

/**
 * @description: set status led on off state
 * @param[in] state: on off state
 * @return: none
 */
STATIC VOID_T status_led_set(BOOL_T onoff)
{
  if (led_info->status_led.driver_flag)
  {
    if (onoff)
    {
      tal_gpio_write(led_info->status_led.pin_num, TUYA_GPIO_LEVEL_HIGH);
    }
    else
    {
      tal_gpio_write(led_info->status_led.pin_num, TUYA_GPIO_LEVEL_LOW);
    }
  }
  else
  {
    if (onoff)
    {
      tal_gpio_write(led_info->status_led.pin_num, TUYA_GPIO_LEVEL_LOW);
    }
    else
    {
      tal_gpio_write(led_info->status_led.pin_num, TUYA_GPIO_LEVEL_HIGH);
    }
  }
}

/**
 * @description: set network led on off state
 * @param[in] state: on off state
 * @return: none
 */
STATIC VOID_T network_led_set(BOOL_T onoff)
{
  if (led_info->net_led.driver_flag)
  {
    if (onoff)
    {
      tal_gpio_write(led_info->net_led.pin_num, TUYA_GPIO_LEVEL_HIGH);
    }
    else
    {
      tal_gpio_write(led_info->net_led.pin_num, TUYA_GPIO_LEVEL_LOW);
    }
  }
  else
  {
    if (onoff)
    {
      tal_gpio_write(led_info->net_led.pin_num, TUYA_GPIO_LEVEL_LOW);
    }
    else
    {
      tal_gpio_write(led_info->net_led.pin_num, TUYA_GPIO_LEVEL_HIGH);
    }
  }
}
