#include "tal_memory.h"

#include "app_common.h"
#include "app_key.h"
#include "app_mfg.h"
#include "bsp.h"

/********************************************************************************
 * @brief Sends a relay time signal and device identity information via radio.
 * @author ysl
 * @return void
 * @date 2025.07
 *
 * This function updates the device information by performing the following tasks:
 * - Sends a relay time signal with a predefined value (0x55) to the radio module.
 * - Introduces a delay of 50 ms to ensure proper timing between operations.
 * - Sends the device ID and type information over the radio using the PRINT_TYPE format.
 ********************************************************************************/
static const char *get_direction_str(UINT8_T dir)
{
  switch (dir)
  {
  case ZG_ZCL_DATA_CLIENT_TO_SERVER:
    return "ZG_ZCL_DATA_CLIENT_TO_SERVER";
  case ZG_ZCL_DATA_SERVER_TO_CLIENT:
    return "ZG_ZCL_DATA_SERVER_TO_CLIENT";
  default:
    return "UNKNOWN_DIRECTION";
  }
}

/********************************************************************************
 * @brief Converts direction enum value to a descriptive string.
 * @author ysl
 * @return const char* - Human-readable direction string
 * @date 2025.07
 *
 * Maps a ZCL data direction value to its corresponding text:
 * - Supports CLIENT_TO_SERVER and SERVER_TO_CLIENT
 * - Returns "UNKNOWN_DIRECTION" for unrecognized values
 ********************************************************************************/
static const char *get_frame_type_str(UINT8_T ft)
{
  switch (ft)
  {
  case ZG_ZCL_FRAME_TYPE_GLOBAL:
    return "ZG_ZCL_FRAME_TYPE_GLOBAL";
  case ZG_ZCL_FRAME_TYPE_SPEC_TO_CLUSTER:
    return "ZG_ZCL_FRAME_TYPE_SPEC_TO_CLUSTER";
  default:
    return "UNKNOWN_FRAME_TYPE";
  }
}

/********************************************************************************
 * @brief Converts frame type enum value to a descriptive string.
 * @author ysl
 * @return const char* - Human-readable frame type string
 * @date 2025.07
 *
 * Maps a ZCL frame type to its corresponding description:
 * - Supports GLOBAL and SPEC_TO_CLUSTER frame types
 * - Returns "UNKNOWN_FRAME_TYPE" for unrecognized values
 ********************************************************************************/
static const char *get_addr_mode_str(UINT8_T mode)
{
  switch (mode)
  {
  case SEND_MODE_DEV:
    return "SEND_MODE_DEV";
  case SEND_MODE_GROUP:
    return "SEND_MODE_GROUP";
  case SEND_MODE_BINDING:
    return "SEND_MODE_BINDING";
  default:
    return "UNKNOWN_ADDR_MODE";
  }
}

/********************************************************************************
 * @brief Converts Zigbee address mode enum to a descriptive string.
 * @author ysl
 * @return const char* - Human-readable address mode string
 * @date 2025.07
 *
 * Maps a send mode value to its readable name:
 * - Supports device, group, and binding modes
 * - Returns "UNKNOWN_ADDR_MODE" if input is invalid
 ********************************************************************************/
VOID_T user_print_send_data(const TAL_ZG_SEND_DATA_T *sd)
{
  if (!sd)
    return;

  TAL_PR_TRACE("========= TAL_ZG_SEND_DATA_T =========");
  TAL_PR_TRACE("zcl_id        : 0x%04X", sd->zcl_id);
  TAL_PR_TRACE("qos           : %d", sd->qos);

  TAL_PR_TRACE("cluster_id    : 0x%04X", sd->addr.type.dev.cluster_id);
  TAL_PR_TRACE("direction     : %s", get_direction_str(sd->direction));
  TAL_PR_TRACE("frame_type    : %s", get_frame_type_str(sd->frame_type));
  TAL_PR_TRACE("command_id    : 0x%02X", sd->command_id);

  TAL_PR_TRACE("delay_time    : %d ms", sd->delay_time);
  TAL_PR_TRACE("random_time   : %d ms", sd->random_time);

  TAL_PR_TRACE("addr.mode     : %s", get_addr_mode_str(sd->addr.mode));
  TAL_PR_TRACE("dst_addr      : 0x%04X", sd->addr.type.dev.dst_addr);
  TAL_PR_TRACE("dst_ep        : %d", sd->addr.type.dev.dst_ep);
  TAL_PR_TRACE("src_ep        : %d", sd->addr.type.dev.src_ep);

  TAL_PR_TRACE("data.len      : %d", sd->data.private.len);

  // if (sd->data.private.len > 0 && sd->data.private.data) {
  //     int len = sd->data.private.len;
  //     if (len > 40) len = 40;  // ӡǰ40ֽ
  //     for (int i = 0; i < len; ++i) {
  //         TAL_PR_TRACE("data[%02d]     : 0x%02X", i, sd->data.private.data[i]);
  //     }
  // }

  // TAL_PR_TRACE("attr_sum        : %d", sd->data.zg.attr_sum);
  // TAL_PR_TRACE("attr_id        : %d", sd->data.zg.attr_id);
  // TAL_PR_TRACE("type        : %d", sd->data.zg.type);
  // TAL_PR_TRACE("size        : %d", sd->data.zg.size);
  // TAL_PR_TRACE("value[0]        : %d", sd->data.zg.value[0]);

  TAL_PR_TRACE("======================================");
}

/********************************************************************************
 * @brief Prints detailed information about a Zigbee send data structure.
 * @author ysl
 * @return void
 * @date 2025.07
 *
 * Used for debugging the TAL_ZG_SEND_DATA_T structure:
 * - Logs basic fields like cluster ID, direction, frame type, and command ID
 * - Outputs addressing information and payload length
 * - Helps track and verify outgoing Zigbee transmissions
 ********************************************************************************/
VOID_T print_zcl_msg_info(const TAL_ZCL_MSG_T *msg)
{
    if (msg == NULL) {
        TAL_PR_ERR("ZCL message is NULL");
        return;
    }

    TAL_PR_TRACE("---- TAL_ZCL_MSG_T ----");
    TAL_PR_TRACE("mode        : %d", msg->mode);
    TAL_PR_TRACE("direction   : %d", msg->direction);
    TAL_PR_TRACE("src_ep      : %d", msg->src_ep);
    TAL_PR_TRACE("dst_ep      : %d", msg->dst_ep);
    TAL_PR_TRACE("zcl_seq     : %d", msg->zcl_seq);
    TAL_PR_TRACE("command     : 0x%02X", msg->command);
    TAL_PR_TRACE("length      : %d", msg->length);
    TAL_PR_TRACE("cluster     : 0x%04X", msg->cluster);
    TAL_PR_TRACE("src_addr    : 0x%04X", msg->src_addr);
    TAL_PR_TRACE("group_id    : 0x%04X", msg->group_id);

    if (msg->payload && msg->length > 0) {
        TAL_PR_TRACE("payload     :");
        for (UINT8_T i = 0; i < msg->length; ++i) {
            TAL_PR_TRACE("%02X ", msg->payload[i]);  // дӡ
        }
        TAL_PR_TRACE("");  // 
    } else {
        TAL_PR_TRACE("payload     : NULL or empty");
    }
}