/**
 * @file tkl_adc.c
 * @brief Common process - adapter the adc api
 * @version 0.1
 * @date 2021-08-06
 *
 * @copyright Copyright 2018-2021 Tuya Inc. All Rights Reserved.
 *
 */

#include "tl_common.h"
#include "tkl_adc.h"
#include "tkl_memory.h"


#define TKL_ADC_DBG 0

#if TKL_ADC_DBG
#include "tkl_uart.h"
#include "tlprintf.h"
#define tkl_adc_debug(...)    tkl_dbg_printf(__VA_ARGS__)
#else
#define tkl_adc_debug(...)
#endif

// const GPIO_PinTypeDef ADC_GPIO_tab[10] = {
// 		GPIO_PB0,GPIO_PB1,
// 		GPIO_PB2,GPIO_PB3,
// 		GPIO_PB4,GPIO_PB5,
// 		GPIO_PB6,GPIO_PB7,
// 		GPIO_PC4,GPIO_PC5
// };

extern GPIO_PinTypeDef ADC_GPIO_tab[10];
STATIC BOOL_T tkl_adc_init_flag = FALSE;
STATIC TUYA_ADC_BASE_CFG_T tkl_adc_cfg = {
    .ch_nums = 1,
    .type = TUYA_ADC_EXTERNAL_SAMPLE_VOL,
    .width = 12,
};


/**
 * @brief tuya kernel adc init
 * 
 * @param[in] port: adc port
 * @param[in] cfg: adc config
 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
TUYA_WEAK_ATTRIBUTE	OPERATE_RET tkl_adc_init(TUYA_ADC_NUM_E port_num, TUYA_ADC_BASE_CFG_T *cfg)
{
	UINT8_T index=0;
	UINT8_T channel_nums=0;

	if((TUYA_ADC_NUM_0!=port_num) || (NULL==cfg)){
        return OPRT_NOT_SUPPORTED;
	}
	
	if(TUYA_ADC_INNER_SAMPLE_VOL==cfg->type){
		// drv_adc_init();
		// drv_adc_mode_pin_set(DRV_ADC_VBAT_MODE, GPIO_PC5);
		// drv_adc_enable(0);
	}
	else if(TUYA_ADC_EXTERNAL_SAMPLE_VOL==cfg->type){
		if(cfg->ch_nums>10){
			tkl_adc_debug("cfg->ch_nums>10\r\n");
			return OPRT_NOT_SUPPORTED;
		}
		else{
			for(index=0;index<10;index++){
				if(cfg->ch_list.data & BIT(index)){
					channel_nums++;
				}
			}
			if(channel_nums && channel_nums==cfg->ch_nums)
			{
				// drv_adc_init();
				// for(index=0;index<10;index++){
				// 	if(cfg->ch_list.data & BIT(index)){
				// 		drv_adc_mode_pin_set(DRV_ADC_BASE_MODE, ADC_GPIO_tab[index]);
				// 	}
				// }
				// drv_adc_enable(0);
			}
			else{
				tkl_adc_debug("channel_nums %d ERROR\r\n",channel_nums);
				return OPRT_NOT_SUPPORTED;
			}
		}
	}

	tkl_system_memcpy((UINT8_T*)&tkl_adc_cfg,(UINT8_T*)cfg,sizeof(TUYA_ADC_BASE_CFG_T));
	tkl_adc_init_flag = TRUE;
    return OPRT_OK;
}

/**
 * @brief adc deinit
 * 
 * @param[in] port: adc port

 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
TUYA_WEAK_ATTRIBUTE	OPERATE_RET tkl_adc_deinit(TUYA_ADC_NUM_E port_num)
{
	if(TUYA_ADC_NUM_0!=port_num){
        return OPRT_NOT_SUPPORTED;
	}

	tkl_adc_init_flag = FALSE;
    return OPRT_OK;
}


/**
 * @brief get adc width
 * 
 * @param[in] port: adc port

 *
 * @return adc width
 */
TUYA_WEAK_ATTRIBUTE UINT8_T tkl_adc_width_get(TUYA_ADC_NUM_E port_num)
{
	if(TUYA_ADC_NUM_0!=port_num){
        return OPRT_NOT_SUPPORTED;
	}

    return tkl_adc_cfg.width;
}


/**
 * @brief get adc reference voltage
 * 
 * @param[in] port: adc port

 *
 * @return adc reference voltage(bat: mv)
 */
TUYA_WEAK_ATTRIBUTE UINT32_T tkl_adc_ref_voltage_get(TUYA_ADC_NUM_E port_num)
{
	if(TUYA_ADC_NUM_0!=port_num){
        return OPRT_NOT_SUPPORTED;
	}

    return tkl_adc_cfg.ref_vol;
}

/**
 * @brief adc read
 * 
 * @param[in] port: adc port
 * @param[out] buff: points to the list of data read from the ADC register
 * @param[out] len:  buff len
 *
 * @return OPRT_OK on success. Others on error, please refer to tuya_error_code.h
 */
TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_adc_read_data(TUYA_ADC_NUM_E port_num, INT32_T *buff, UINT16_T len)
{
    return OPRT_NOT_SUPPORTED;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_adc_read_voltage(TUYA_ADC_NUM_E port_num, INT32_T *buff, UINT16_T len)
{
    return OPRT_NOT_SUPPORTED;
}


TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_adc_read_single_channel(TUYA_ADC_NUM_E port_num, UINT8_T ch_id, INT32_T *data)
{
    UINT16_T adc;

	if(FALSE==tkl_adc_init_flag){
		return OPRT_NOT_SUPPORTED;
	}

    if(ch_id >= 10){
        return OPRT_NOT_SUPPORTED;
    }

    if(!data){
        return OPRT_INVALID_PARM;    
    }

	if(TUYA_ADC_INNER_SAMPLE_VOL==tkl_adc_cfg.type){
		drv_adc_init();
		drv_adc_mode_pin_set(DRV_ADC_VBAT_MODE, GPIO_PC5);
		drv_adc_enable(1);
		adc = drv_get_adc_data();
		tkl_adc_debug("drv_get_adc_data adc:%d\r\n",adc);
		*data = (INT32_T)adc;
		drv_adc_enable(0);		
	}
	else if(TUYA_ADC_EXTERNAL_SAMPLE_VOL==tkl_adc_cfg.type){
		if(!(tkl_adc_cfg.ch_list.data & BIT(ch_id))){
			return OPRT_NOT_SUPPORTED;
		}
		drv_adc_init();
		drv_adc_mode_pin_set(DRV_ADC_BASE_MODE, ADC_GPIO_tab[ch_id]);
		drv_adc_enable(1);
		adc = drv_get_adc_data();
		tkl_adc_debug("drv_get_adc_data adc:%d\r\n",adc);
		*data = (INT32_T)adc;
		drv_adc_enable(0);
	}

    return OPRT_OK;
}


/******************************************************************************/
/**                              EOF                                         **/
/******************************************************************************/
