/**
 * @file tkl_gpio.c
 * @brief Common process - adapter the gpio api
 * @version 0.1
 * @date 2021-08-06
 *
 * @copyright Copyright 2018-2021 Tuya Inc. All Rights Reserved.
 *
 */

#include "tl_common.h"
#include "tkl_gpio.h"
#include "tkl_memory.h"
#include "gpio.h"
#include "register.h"
#include "tkl_platform_types.h"
#include "tal_log.h"



#define TKL_GPIO_DBG 0

#if TKL_GPIO_DBG
#include "tkl_uart.h"
#include "tlprintf.h"
#define tkl_gpio_debug(...)    tkl_dbg_printf(__VA_ARGS__)
#else
#define tkl_gpio_debug(...)
#endif

#define SUPPORT_MAX_GPIO    16
#define INT_INVALID         0xFF

typedef void (*tl_gpio_int_func_t)(void);

typedef struct{
    UINT32_T pin;
    BOOL_T  enable;
    tl_gpio_int_func_t tl_func;
    TUYA_GPIO_IRQ_T cfg;
    UINT32_T falling;
}irq_src_arr;

irq_src_arr irq_src_mask[SUPPORT_MAX_GPIO] = {};
TUYA_GPIO_MODE_E gpio_mode_mask[SUPPORT_MAX_GPIO] = {};

STATIC CONST UINT32_T tkl_gpio_list[] = {
    GPIO_PA0, GPIO_PA1, GPIO_PA2, GPIO_PA3, GPIO_PA4, GPIO_PA5, GPIO_PA6, GPIO_PA7, // 0 - 7 //
    GPIO_PB0, GPIO_PB1, GPIO_PB2, GPIO_PB3, GPIO_PB4, GPIO_PB5, GPIO_PB6, GPIO_PB7, // 8 - 15 //
    GPIO_PC0, GPIO_PC1, GPIO_PC2, GPIO_PC3, GPIO_PC4, GPIO_PC5, GPIO_PC6, GPIO_PC7, // 16 - 23 //
    GPIO_PD0, GPIO_PD1, GPIO_PD2, GPIO_PD3, GPIO_PD4, GPIO_PD5, GPIO_PD6, GPIO_PD7, // 24 - 31 //
    GPIO_PE0, GPIO_PE1, GPIO_PE2, GPIO_PE3, // 32 - 35//
};

extern void tkl_pwm_func_recovery(void);
extern void tkl_uart_func_recovery(void);

#if END_DEVICE
extern VOID hardware_set_wakeup(UINT32_T bit_map);
extern VOID hardware_clear_wakeup(UINT32_T bit_map);
#endif


TUYA_WEAK_ATTRIBUTE OPERATE_RET check_pin_register(TUYA_GPIO_NUM_E pin_id, UINT8_T *index, drv_gpioIrqMode_t *mode);

TUYA_WEAK_ATTRIBUTE void tkl_gpio_special_func_recovery(void)
{
	tkl_pwm_func_recovery();
    tkl_uart_func_recovery();
}


TUYA_WEAK_ATTRIBUTE BOOL_T tkl_gpio_output_use_wakeup_mode(TUYA_GPIO_NUM_E pin_id)
{
  // TAL_PR_TRACE("tkl_gpio_output_use_wakeup_mode FALSE");
    return FALSE;
}

TUYA_WEAK_ATTRIBUTE GPIO_PullTypeDef tkl_gpio_output_pull_config(TUYA_GPIO_NUM_E pin_id,TUYA_GPIO_LEVEL_E level)
{
    return PM_PIN_UP_DOWN_FLOAT;
}

TUYA_WEAK_ATTRIBUTE BOOL_T tkl_reallocate_gpio_interrupt_source(TUYA_GPIO_NUM_E pin,UINT8_T before_mode,UINT8_T* after_mode)
{
    return FALSE;
}

TUYA_WEAK_ATTRIBUTE void dev_gpio_enter_sleep(void)
{
#if END_DEVICE
    hardware_clear_wakeup(DEV_GPIO_WAKE_UP_BITMAPS);
#endif
}

TUYA_WEAK_ATTRIBUTE void dev_gpio_enter_wakeup(void)
{
#if END_DEVICE
    hardware_set_wakeup(DEV_GPIO_WAKE_UP_BITMAPS);
#endif
}

TUYA_WEAK_ATTRIBUTE UINT32_T tkl_gpio_to_tlsr_gpio(UINT32_T pin_id){
    if(pin_id >= (sizeof(tkl_gpio_list)/sizeof(tkl_gpio_list[0]))){
        return 0xFFFFFFFF;
    }
    UINT32_T gpio_tlsr = tkl_gpio_list[pin_id];
    return gpio_tlsr;
}

TUYA_WEAK_ATTRIBUTE UINT32_T tlsr_gpio_to_tkl_gpio(UINT32_T gpio_tlsr){
    UINT32_T pin_id;
    for(int i = 0; i < (sizeof(tkl_gpio_list)/sizeof(tkl_gpio_list[0])); i++){
        if(gpio_tlsr == tkl_gpio_list[i]){
            return pin_id;
        }
    }

    return 0xFF;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_gpio_init(TUYA_GPIO_NUM_E pin_id, CONST TUYA_GPIO_BASE_CFG_T *cfg){
    OPERATE_RET ret = OPRT_OK;
    UINT32_T gpio_tlsr = tkl_gpio_to_tlsr_gpio(pin_id);
    drv_gpioIrqMode_t pin_mode = GPIO_IRQ_MODE;
    UINT8_T i = 0;

    if(cfg == NULL) {
        return OPRT_INVALID_PARM;
    }

    if(OPRT_OK!=check_pin_register(pin_id,&i,&pin_mode)){
        return OPRT_COM_ERROR;
    }

    gpio_mode_mask[i] = cfg->mode;

    switch(cfg->mode){
        case TUYA_GPIO_PULLUP:
            gpio_setup_up_down_resistor(gpio_tlsr, PM_PIN_PULLUP_10K);
        break;    
        case TUYA_GPIO_PULLDOWN:
            gpio_setup_up_down_resistor(gpio_tlsr, PM_PIN_PULLDOWN_100K);
        break;
        case TUYA_GPIO_FLOATING:
        case TUYA_GPIO_HIGH_IMPEDANCE:
            gpio_setup_up_down_resistor(gpio_tlsr, PM_PIN_UP_DOWN_FLOAT);
        break;
        case TUYA_GPIO_PUSH_PULL:
        case TUYA_GPIO_OPENDRAIN:
        case TUYA_GPIO_OPENDRAIN_PULLUP:
            gpio_setup_up_down_resistor(gpio_tlsr, tkl_gpio_output_pull_config(pin_id,cfg->level));
        break;
        default:
            gpio_setup_up_down_resistor(gpio_tlsr, PM_PIN_UP_DOWN_FLOAT);
        break;
    }

    if(TUYA_GPIO_INPUT == cfg->direct){
        gpio_set_output_en(gpio_tlsr, 0);
        gpio_set_input_en(gpio_tlsr, 1);
    }
    else if(TUYA_GPIO_OUTPUT == cfg->direct){
        gpio_set_output_en(gpio_tlsr, 1);
        gpio_set_input_en(gpio_tlsr, 0);
        gpio_write(gpio_tlsr, (cfg->level == TUYA_GPIO_LEVEL_HIGH)?1:0);
    }
    else{
        return OPRT_NOT_SUPPORTED;
    }

    
    gpio_set_func(gpio_tlsr, AS_GPIO);

    return ret;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_gpio_deinit(TUYA_GPIO_NUM_E pin_id){ 
#if (PM_ENABLE)
    return OPRT_OK;
#else
    UINT32_T gpio_tlsr = tkl_gpio_to_tlsr_gpio(pin_id);
    gpio_shutdown(gpio_tlsr);
    return OPRT_OK;
#endif
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_gpio_write(TUYA_GPIO_NUM_E pin_id, TUYA_GPIO_LEVEL_E level){
    UINT32_T gpio_tlsr = tkl_gpio_to_tlsr_gpio(pin_id);
    
    gpio_set_func(gpio_tlsr, AS_GPIO);
    gpio_set_output_en(gpio_tlsr, 1); 			//enable output
	gpio_set_input_en(gpio_tlsr, 0);			//disable input

#if END_DEVICE
    if(TUYA_GPIO_LEVEL_HIGH==level){
        if(!tkl_gpio_output_use_wakeup_mode(pin_id)) {
            gpio_set_output_en(gpio_tlsr, 0);
            gpio_setup_up_down_resistor(gpio_tlsr, PM_PIN_PULLUP_10K);
        } else {
            gpio_setup_up_down_resistor(gpio_tlsr, tkl_gpio_output_pull_config(pin_id,level));
            gpio_set_output_en(gpio_tlsr, 1);
            gpio_write(gpio_tlsr, 1);
        }
    }
    else{
        if(!tkl_gpio_output_use_wakeup_mode(pin_id)) {
            gpio_set_output_en(gpio_tlsr, 0);
            gpio_setup_up_down_resistor(gpio_tlsr, PM_PIN_PULLUP_1M);
            gpio_setup_up_down_resistor(gpio_tlsr, PM_PIN_PULLDOWN_100K);
        } else {
            gpio_setup_up_down_resistor(gpio_tlsr, tkl_gpio_output_pull_config(pin_id,level));
            gpio_set_output_en(gpio_tlsr, 1);
            gpio_write(gpio_tlsr, 0);
        }        
    }
#else
    gpio_write(gpio_tlsr, level?1:0);
#endif
    return OPRT_OK;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_gpio_read(TUYA_GPIO_NUM_E pin_id, TUYA_GPIO_LEVEL_E *level){
    UINT32_T gpio_tlsr = tkl_gpio_to_tlsr_gpio(pin_id);
    if(level == NULL) {
        return OPRT_INVALID_PARM;
    }

    gpio_set_func(gpio_tlsr, AS_GPIO);
	gpio_set_output_en(gpio_tlsr, 0); 			//disable output
	gpio_set_input_en(gpio_tlsr, 1);			//enable input

    *level = gpio_read(gpio_tlsr)?TUYA_GPIO_LEVEL_HIGH:TUYA_GPIO_LEVEL_LOW;
    return OPRT_OK;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET check_pin_register(TUYA_GPIO_NUM_E pin_id, UINT8_T *index, drv_gpioIrqMode_t *mode){
    UINT8_T i = 0xFF;
    OPERATE_RET ret = OPRT_COM_ERROR;
    UINT8_T input = 0;

    if(!index){
        return OPRT_COM_ERROR;
    }
    for(i=0; i<SUPPORT_MAX_GPIO; i++) {
        if((irq_src_mask[i].pin == pin_id) && (irq_src_mask[i].pin != INT_INVALID)){
            *index = i;
            *mode = (drv_gpioIrqMode_t)(i%3+1);
            if(tkl_reallocate_gpio_interrupt_source(pin_id,*(UINT8_T*)mode,&input)){
                if(input<=GPIO_IRQ_RISC1_MODE && input){
                    *mode = input;
                }  
            }
            return OPRT_OK;
        }
    }

    for(i=0; i<SUPPORT_MAX_GPIO; i++) {
        if((irq_src_mask[i].enable == FALSE) && (irq_src_mask[i].pin == INT_INVALID)){
            *index = i;
            *mode = (drv_gpioIrqMode_t)(i%3+1);
            if(tkl_reallocate_gpio_interrupt_source(pin_id,*(UINT8_T*)mode,&input)){
                if(input<=GPIO_IRQ_RISC1_MODE && input){
                    *mode = input;
                }  
            }
            return OPRT_OK;
        }
    }
    return ret;
}


static void dev_gpio_trigger_all_handle(UINT8_T x)
{
    UINT8_T level = 0;
    BOOL_T exe_flag = TRUE;
    UINT32_T gpio_tlsr = 0;
    drv_gpioIrqMode_t pin_mode = (drv_gpioIrqMode_t)(x%3+1);
    drv_gpioIrqMode_t input_mode = GPIO_IRQ_INVALID;
    drv_pm_pinCfg_t wake_pinCfg;

    if(x >= SUPPORT_MAX_GPIO){
        return;
    }
    if((irq_src_mask[x].enable == FALSE) || (irq_src_mask[x].pin == INT_INVALID)){
        return;
    }
    if(OPRT_OK!=tkl_gpio_read(irq_src_mask[x].pin,&level)){
        return;
    }
    
    if(TUYA_GPIO_IRQ_RISE == irq_src_mask[x].cfg.mode){
        if(!level){
            exe_flag = FALSE;
        }
    }
    else if(TUYA_GPIO_IRQ_FALL == irq_src_mask[x].cfg.mode){
        if(level){
            exe_flag = FALSE;
        }
    }

    if(level){
        irq_src_mask[x].falling = FALLING_EDGE;
    }else{
        irq_src_mask[x].falling = RISING_EDGE;
    }

    gpio_tlsr = tkl_gpio_to_tlsr_gpio(irq_src_mask[x].pin);


    if(tkl_reallocate_gpio_interrupt_source(irq_src_mask[x].pin,pin_mode,(UINT8_T*)&input_mode)){
        if(input_mode<=GPIO_IRQ_RISC1_MODE && input_mode){
            pin_mode = input_mode;
        }  
    }

    drv_gpio_irq_config(pin_mode, gpio_tlsr, irq_src_mask[x].falling, irq_src_mask[x].tl_func);

    wake_pinCfg.wakeupPin = gpio_tlsr;
    (level)?(wake_pinCfg.wakeupLevel=PM_WAKEUP_LEVEL_LOW):(wake_pinCfg.wakeupLevel=PM_WAKEUP_LEVEL_HIGH);
#if (PM_ENABLE)
    drv_pm_wakeupPinConfig(&wake_pinCfg, 1);
    tkl_gpio_debug("wake pin config result:%d \r\n",drv_pm_wakeupPinValid(&wake_pinCfg, 1));
#endif

    if(exe_flag){
        if(irq_src_mask[x].cfg.cb){
            irq_src_mask[x].cfg.cb(irq_src_mask[x].cfg.arg);
        }        
    }
}


TUYA_WEAK_ATTRIBUTE void dev_gpio_trigger_all_without_callback(TUYA_GPIO_NUM_E pin)
{
    UINT32_T gpio_tlsr = tkl_gpio_to_tlsr_gpio(pin);
    drv_gpioIrqMode_t pin_mode = GPIO_IRQ_MODE;
    drv_gpioIrqMode_t input_mode = GPIO_IRQ_INVALID;
    TUYA_GPIO_LEVEL_E level = TUYA_GPIO_LEVEL_LOW;
    drv_pm_pinCfg_t wake_pinCfg;
    UINT8_T i = 0;

    if(OPRT_OK!=check_pin_register(pin,&i,&pin_mode)){
        return;
    }

    if((irq_src_mask[i].enable == FALSE) || (irq_src_mask[i].pin == INT_INVALID)){
        return;
    }

    if(OPRT_OK!=tkl_gpio_read(irq_src_mask[i].pin,&level)){
        return;
    }

    if(level){
        irq_src_mask[i].falling = FALLING_EDGE;
    }else{
        irq_src_mask[i].falling = RISING_EDGE;
    }

    gpio_tlsr = tkl_gpio_to_tlsr_gpio(irq_src_mask[i].pin);


    if(tkl_reallocate_gpio_interrupt_source(irq_src_mask[i].pin,pin_mode,(UINT8_T*)&input_mode)){
        if(input_mode<=GPIO_IRQ_RISC1_MODE && input_mode){
            pin_mode = input_mode;
        }  
    }

    drv_gpio_irq_config(pin_mode, gpio_tlsr, irq_src_mask[i].falling, irq_src_mask[i].tl_func);

    wake_pinCfg.wakeupPin = gpio_tlsr;
    (level)?(wake_pinCfg.wakeupLevel=PM_WAKEUP_LEVEL_LOW):(wake_pinCfg.wakeupLevel=PM_WAKEUP_LEVEL_HIGH);
#if (PM_ENABLE)
    drv_pm_wakeupPinConfig(&wake_pinCfg, 1);
    tkl_gpio_debug("wake pin config result:%d \r\n",drv_pm_wakeupPinValid(&wake_pinCfg, 1));
#endif

}

#define GPIO_INT_HANDLER(x) \
void __gpio_int_cb_##x(void) { \
    dev_gpio_trigger_all_handle(x); \
}

GPIO_INT_HANDLER(0);
GPIO_INT_HANDLER(1);
GPIO_INT_HANDLER(2);
GPIO_INT_HANDLER(3);
GPIO_INT_HANDLER(4);
GPIO_INT_HANDLER(5);
GPIO_INT_HANDLER(6);
GPIO_INT_HANDLER(7);
GPIO_INT_HANDLER(8);
GPIO_INT_HANDLER(9);
GPIO_INT_HANDLER(10);
GPIO_INT_HANDLER(11);
GPIO_INT_HANDLER(12);
GPIO_INT_HANDLER(13);
GPIO_INT_HANDLER(14);
GPIO_INT_HANDLER(15);


TUYA_WEAK_ATTRIBUTE void gpio_input_init(void)
{
    UINT8_T i = 0;
    
    irq_src_mask[i++].tl_func = __gpio_int_cb_0;
    irq_src_mask[i++].tl_func = __gpio_int_cb_1;
    irq_src_mask[i++].tl_func = __gpio_int_cb_2;
    irq_src_mask[i++].tl_func = __gpio_int_cb_3;
    irq_src_mask[i++].tl_func = __gpio_int_cb_4;
    irq_src_mask[i++].tl_func = __gpio_int_cb_5;
    irq_src_mask[i++].tl_func = __gpio_int_cb_6;
    irq_src_mask[i++].tl_func = __gpio_int_cb_7;
    irq_src_mask[i++].tl_func = __gpio_int_cb_8;
    irq_src_mask[i++].tl_func = __gpio_int_cb_9;
    irq_src_mask[i++].tl_func = __gpio_int_cb_10;
    irq_src_mask[i++].tl_func = __gpio_int_cb_11;
    irq_src_mask[i++].tl_func = __gpio_int_cb_12;
    irq_src_mask[i++].tl_func = __gpio_int_cb_13;
    irq_src_mask[i++].tl_func = __gpio_int_cb_14;
    irq_src_mask[i++].tl_func = __gpio_int_cb_15;

    for(i=0; i<SUPPORT_MAX_GPIO; i++) {
        irq_src_mask[i].pin = INT_INVALID;
        irq_src_mask[i].enable = FALSE;
        irq_src_mask[i].cfg.mode = GPIO_MODE_INPUT_HIGH_IMPEDANCE;
        irq_src_mask[i].cfg.cb = NULL;
        irq_src_mask[i].cfg.arg = NULL;
        irq_src_mask[i].falling = RISING_FALLING_EDGE;
        gpio_mode_mask[i] = 0xFF;
    }
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_inut_gpio_mode_recover(TUYA_GPIO_NUM_E pin_id, CONST TUYA_GPIO_IRQ_T *cfg){
    return OPRT_OK;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_gpio_irq_init(TUYA_GPIO_NUM_E pin_id, CONST TUYA_GPIO_IRQ_T *cfg){
    UINT32_T gpio_tlsr = tkl_gpio_to_tlsr_gpio(pin_id);
    drv_gpioIrqMode_t pin_mode = GPIO_IRQ_MODE;
    UINT8_T i = 0;
    TUYA_GPIO_LEVEL_E level = TUYA_GPIO_LEVEL_LOW;
    TUYA_GPIO_BASE_CFG_T init_param;

    if((gpio_tlsr==0xFFFFFFFF) || (!cfg)) {
        return OPRT_COM_ERROR;    
    }

    if(OPRT_OK!=check_pin_register(pin_id,&i,&pin_mode)){
        return OPRT_COM_ERROR;
    }
    TAL_PR_TRACE("irq_src_mask[%d].falling:%d", i, irq_src_mask[i].falling);
    TAL_PR_TRACE("cfg->mode:%d", cfg->mode);
    if(RISING_FALLING_EDGE==irq_src_mask[i].falling){//first init
        switch(cfg->mode){
            case TUYA_GPIO_IRQ_RISE:
                init_param.mode = TUYA_GPIO_PULLDOWN;
                init_param.level = TUYA_GPIO_LEVEL_LOW;
                irq_src_mask[i].falling = RISING_EDGE;
            break;
            case TUYA_GPIO_IRQ_FALL:
                init_param.mode = TUYA_GPIO_PULLUP;
                init_param.level = TUYA_GPIO_LEVEL_HIGH;
                irq_src_mask[i].falling = FALLING_EDGE;
            break;
            case TUYA_GPIO_IRQ_RISE_FALL:
                if(OPRT_OK==tkl_gpio_read(pin_id,&level)){
                    if(TUYA_GPIO_LEVEL_LOW==level){
                        init_param.mode = TUYA_GPIO_PULLDOWN;
                        init_param.level = TUYA_GPIO_LEVEL_LOW;
                        irq_src_mask[i].falling = RISING_EDGE;
                    }
                    else if(TUYA_GPIO_LEVEL_HIGH==level){
                        init_param.mode = TUYA_GPIO_PULLUP;
                        init_param.level = TUYA_GPIO_LEVEL_HIGH;
                        irq_src_mask[i].falling = FALLING_EDGE;
                    }
                } 
            break;
            default:
                return OPRT_NOT_SUPPORTED;
            break;
        }
    }
    else{
        if(RISING_EDGE==irq_src_mask[i].falling){
            init_param.mode = TUYA_GPIO_PULLDOWN;
            init_param.level = TUYA_GPIO_LEVEL_LOW;
        }
        else if(FALLING_EDGE==irq_src_mask[i].falling){
            init_param.mode = TUYA_GPIO_PULLUP;
            init_param.level = TUYA_GPIO_LEVEL_HIGH;
        }
    }

    if(0xFF!=gpio_mode_mask[i]){
        init_param.mode = gpio_mode_mask[i];
    }

    init_param.direct = TUYA_GPIO_INPUT;
    tkl_gpio_init(pin_id,&init_param);

#if (PM_ENABLE)
    drv_pm_pinCfg_t wake_pinCfg;
    wake_pinCfg.wakeupPin = gpio_tlsr;
    (init_param.level==TUYA_GPIO_LEVEL_HIGH)?(wake_pinCfg.wakeupLevel=PM_WAKEUP_LEVEL_LOW):(wake_pinCfg.wakeupLevel=PM_WAKEUP_LEVEL_HIGH);
    drv_pm_wakeupPinConfig(&wake_pinCfg, 1);
    tkl_gpio_debug("tkl_gpio_init wake pin config result:%d wake_pinCfg.wakeupLevel:%d\r\n",drv_pm_wakeupPinValid(&wake_pinCfg, 1),wake_pinCfg.wakeupLevel);
#endif

    TAL_PR_TRACE("drv_gpio_irq_conf pin:%x,falling:%x \r\n",gpio_tlsr,irq_src_mask[i].falling);
    drv_gpio_irq_config(pin_mode, gpio_tlsr, irq_src_mask[i].falling, irq_src_mask[i].tl_func);

    irq_src_mask[i].enable = TRUE;
    irq_src_mask[i].pin = pin_id;
    tkl_system_memcpy((UINT8_T*)&irq_src_mask[i].cfg,(UINT8_T*)cfg,sizeof(TUYA_GPIO_IRQ_T));

    TAL_PR_TRACE("irq_src_mask enable:%x,pin:%x,args:%x,callback:%x\r\n",\
                    irq_src_mask[i].enable,irq_src_mask[i].pin,irq_src_mask[i].cfg.arg,irq_src_mask[i].cfg.cb);

    if(GPIO_IRQ_RISC0_MODE == pin_mode) {
        gpio_en_interrupt_risc0(gpio_tlsr, 1);
    } else if(GPIO_IRQ_RISC1_MODE == pin_mode) {
        gpio_en_interrupt_risc1(gpio_tlsr, 1);
    } else {
        gpio_en_interrupt(gpio_tlsr, 1);
    }

    return OPRT_OK;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_gpio_irq_enable(TUYA_GPIO_NUM_E pin_id){
    UINT32_T gpio_tlsr = tkl_gpio_to_tlsr_gpio(pin_id);
    UINT8_T i = 0;

    if(gpio_tlsr==0xFFFFFFFF){
        return OPRT_COM_ERROR;    
    }

    for(i=0; i<SUPPORT_MAX_GPIO; i++) {
        if((irq_src_mask[i].pin == pin_id) && (irq_src_mask[i].pin != INT_INVALID)){
            break;
        }
    }

    if(SUPPORT_MAX_GPIO == i){
        return OPRT_COM_ERROR;
    }

    irq_src_mask[i].enable = TRUE;
    tkl_gpio_irq_init(irq_src_mask[i].pin, &irq_src_mask[i].cfg);
    
    return OPRT_OK;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_gpio_irq_disable(TUYA_GPIO_NUM_E pin_id){
    UINT32_T gpio_tlsr = tkl_gpio_to_tlsr_gpio(pin_id);
    drv_gpioIrqMode_t pin_mode = GPIO_IRQ_INVALID;
    drv_gpioIrqMode_t input_mode = GPIO_IRQ_INVALID;
    UINT8_T i = 0;
    
    if(gpio_tlsr==0xFFFFFFFF){
        return OPRT_COM_ERROR;    
    }

    for(i=0; i<SUPPORT_MAX_GPIO; i++) {
        if((irq_src_mask[i].enable == TRUE) && (irq_src_mask[i].pin == pin_id) && \
           (irq_src_mask[i].pin != INT_INVALID)){
            pin_mode = (drv_gpioIrqMode_t)(i%3+1);
            if(tkl_reallocate_gpio_interrupt_source(pin_id,pin_mode,(UINT8_T*)&input_mode)){
                if(input_mode<=GPIO_IRQ_RISC1_MODE && input_mode){
                    pin_mode = input_mode;
                }  
            }
            break;
        }
    }

    if(GPIO_IRQ_RISC0_MODE == pin_mode){
        gpio_en_interrupt_risc0(gpio_tlsr, 0);
    }
    else if(GPIO_IRQ_RISC1_MODE == pin_mode){
        gpio_en_interrupt_risc1(gpio_tlsr, 0);
    }
    else if(GPIO_IRQ_MODE == pin_mode){
        gpio_en_interrupt(gpio_tlsr, 0);
    }
    else{
        return OPRT_COM_ERROR;
    }

    irq_src_mask[i].enable = FALSE;

    return OPRT_OK;
}

TUYA_WEAK_ATTRIBUTE void tkl_inut_gpio_recover_before_sleep(void)
{

}

TUYA_WEAK_ATTRIBUTE void tkl_gpio_irq_recovery(void)
{
    UINT8_T i;    

    for(i=0; i<SUPPORT_MAX_GPIO; i++) {
        if((irq_src_mask[i].enable == TRUE) && (irq_src_mask[i].pin != INT_INVALID)){
            tkl_gpio_irq_init(irq_src_mask[i].pin, &irq_src_mask[i].cfg);
        }
    }

    drv_gpio_irq_handler();
    drv_gpio_irq_risc0_handler();
    drv_gpio_irq_risc1_handler();
}





