/**
 * @file tkl_pwm.c
 * @brief Common process - adapter the pwm api
 * @version 0.1
 * @date 2021-10-11
 *
 * @copyright Copyright 2018-2021 Tuya Inc. All Rights Reserved.
 *
 */


#include "tkl_pwm.h"
#include "tkl_board.h"
#include "pwm.h"
#include "gpio.h"
#include "tkl_platform_types.h"
#include "tkl_memory.h"

// #define TUYA_RUNTIME_DBG  1

#if defined(TUYA_RUNTIME_DBG)
#include "tkl_uart.h"
#include "tlprintf.h"
#define tkl_pwm_debug(...) tkl_dbg_printf(__VA_ARGS__)
#else
#define tkl_pwm_debug(...)
#endif

#define TKL_PWM_MAX_CHANNEL 6

#if END_DEVICE
extern VOID hardware_set_wakeup(UINT32_T bit_map);
extern VOID hardware_clear_wakeup(UINT32_T bit_map);
#endif

extern UINT32_T tkl_gpio_to_tlsr_gpio(UINT32_T pin_id);

TUYA_PWM_BASE_CFG_T tkl_pwm_cfg[TKL_PWM_MAX_CHANNEL] = {{0,0,0,0,0},{0,0,0,0,0},{0,0,0,0,0},{0,0,0,0,0},{0,0,0,0,0},{0,0,0,0,0}};
static UINT8_T  g_pwm_table_index[TKL_PWM_MAX_CHANNEL] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
static UINT8_T g_firt_in[TKL_PWM_MAX_CHANNEL] = {0,0,0,0,0,0};
static UINT16_T g_pwm_precision[TKL_PWM_MAX_CHANNEL] = {1000,1000,1000,1000,1000,1000};
static UINT8_T  g_pwm_idle_level[TKL_PWM_MAX_CHANNEL] = {0,0,0,0,0,0};

typedef struct {
    GPIO_PinTypeDef gpio;
    GPIO_FuncTypeDef func;
    UINT8_T channel;
    UINT8_T invert;
}pwm_gpio_channel_t;

const pwm_gpio_channel_t g_io_pwm_table[] = {
    {GPIO_PA0, AS_PWM0_N, 0, 0}, //0
    {GPIO_PA2, AS_PWM0,   0, 1}, //1
    {GPIO_PA3, AS_PWM1,   1, 1},//2
    {GPIO_PA4, AS_PWM2,   2, 1},//3

    {GPIO_PB0, AS_PWM3,   3, 1},//4
    {GPIO_PB1, AS_PWM4,   4, 1},//5
    {GPIO_PB2, AS_PWM5,   5, 1},//6
    {GPIO_PB3, AS_PWM0_N, 0, 0},//7
    {GPIO_PB4, AS_PWM4,   4, 1},//8
    {GPIO_PB5, AS_PWM5,   5, 1},//9
    
    {GPIO_PC0, AS_PWM4_N, 4, 0},//10
    {GPIO_PC1, AS_PWM1_N, 1, 0},//11
    {GPIO_PC2, AS_PWM0,   0, 1},//12
    {GPIO_PC3, AS_PWM1,   1, 1},//13
    {GPIO_PC4, AS_PWM2,   2, 1},//14
    {GPIO_PC5, AS_PWM3_N, 3, 0},//15
    {GPIO_PC6, AS_PWM4_N, 4, 0},//16
    {GPIO_PC7, AS_PWM5_N, 5, 0},//17

    {GPIO_PD2, AS_PWM3,   3, 1},//18
    {GPIO_PD3, AS_PWM1_N, 1, 0},//19
    {GPIO_PD4, AS_PWM2_N, 2, 0},//20
    {GPIO_PD5, AS_PWM0_N, 0, 0},//21
};

#define get_array_len(x) (sizeof(x)/sizeof(x[0]))
#define IO_PWM_MAP_SUM  get_array_len(g_io_pwm_table)

TUYA_WEAK_ATTRIBUTE UINT8_T get_io_pwm_map_index(GPIO_PinTypeDef pin)
{
    UINT8_T i = 0;
    
    for(i=0; i<IO_PWM_MAP_SUM; i++) {
        if(g_io_pwm_table[i].gpio == pin) {
            return i;
        }
    }

    return 0xFF;
}

TUYA_WEAK_ATTRIBUTE UINT16_T get_pwm_max_tick(UINT_T frequency)
{
    UINT32_T default_ticksPerPeriod = 0;
    UINT32_T g_pwm_default_work_freq = CLOCK_SYS_CLOCK_HZ;

    if(!frequency){
        return 0xFFFF;
    }

    while(1) {
        default_ticksPerPeriod = g_pwm_default_work_freq/frequency;
        if(default_ticksPerPeriod < 65534UL) {
            break;
        }
        g_pwm_default_work_freq = g_pwm_default_work_freq>>1;
    }
    return default_ticksPerPeriod;
}


TUYA_WEAK_ATTRIBUTE OPERATE_RET config_pwm_polarity(TUYA_PWM_NUM_E ch_id)
{
    UINT8_T i = 0;
    
    if(ch_id >= TKL_PWM_MAX_CHANNEL) {
        return OPRT_COM_ERROR;
    }

    i = g_pwm_table_index[ch_id];

    if(g_pwm_idle_level[ch_id]){
        if(TUYA_PWM_POSITIVE == tkl_pwm_cfg[ch_id].polarity){
            tkl_pwm_debug("polarity ch id:%d,value:%d \r\n",ch_id,tkl_pwm_cfg[ch_id].polarity);
            if(g_io_pwm_table[i].invert == 1){
                pwm_polo_enable(g_io_pwm_table[i].channel,1);
            }
            else{
                pwm_polo_enable(g_io_pwm_table[i].channel,1);
            }
        }
        else{
            tkl_pwm_debug("polarity ch id:%d,value:%d \r\n",ch_id,tkl_pwm_cfg[ch_id].polarity);
            if(g_io_pwm_table[i].invert == 1){
                pwm_polo_enable(g_io_pwm_table[i].channel,0);
            }
            else{
                pwm_polo_enable(g_io_pwm_table[i].channel,0);
            }
        }   
    }
    else{
        if(TUYA_PWM_POSITIVE == tkl_pwm_cfg[ch_id].polarity){
            tkl_pwm_debug("polarity ch id:%d,value:%d \r\n",ch_id,tkl_pwm_cfg[ch_id].polarity);
            if(g_io_pwm_table[i].invert == 1){
                pwm_polo_enable(g_io_pwm_table[i].channel,0);
            }
            else{
                pwm_polo_enable(g_io_pwm_table[i].channel,0);
            }
        }
        else{
            tkl_pwm_debug("polarity ch id:%d,value:%d \r\n",ch_id,tkl_pwm_cfg[ch_id].polarity);
            if(g_io_pwm_table[i].invert == 1){
                pwm_polo_enable(g_io_pwm_table[i].channel,1);
            }
            else{
                pwm_polo_enable(g_io_pwm_table[i].channel,1);
            }
        }    
    }
    return OPRT_OK;
}


TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_pwm_init(TUYA_PWM_NUM_E ch_id, CONST TUYA_PWM_BASE_CFG_T *cfg)
{
    UINT8_T i = 0;
    UINT32_T default_ticksPerPeriod = 0;
    UINT32_T pwm_max_tick = 0;
    UINT32_T g_pwm_default_work_freq = CLOCK_SYS_CLOCK_HZ;

    if((ch_id>=TKL_PWM_MAX_CHANNEL)||(!cfg)||(!cfg->frequency)){
        return OPRT_COM_ERROR;
    }

    memcpy(&tkl_pwm_cfg[ch_id], cfg, sizeof(TUYA_PWM_BASE_CFG_T));

    i = g_pwm_table_index[ch_id];

    if(i==0xFF){
        return OPRT_COM_ERROR;
    }

    while(1) {
        default_ticksPerPeriod = g_pwm_default_work_freq/cfg->frequency;
        if(default_ticksPerPeriod < 65534UL) {
            break;
        }
        g_pwm_default_work_freq = g_pwm_default_work_freq>>1;
    }

    pwm_set_clk(CLOCK_SYS_CLOCK_HZ, g_pwm_default_work_freq);

    pwm_max_tick  = default_ticksPerPeriod;

    if(g_pwm_idle_level[ch_id]){
        if(TUYA_PWM_POSITIVE == cfg->polarity){
            tkl_pwm_debug("polarity ch id:%d,value:%d \r\n",ch_id,cfg->polarity);
            if(g_io_pwm_table[i].invert == 1){
                pwm_revert(g_io_pwm_table[i].channel);
            }
            else{
                pwm_n_revert(g_io_pwm_table[i].channel);
            }
        }
        else{
            tkl_pwm_debug("polarity ch id:%d,value:%d \r\n",ch_id,cfg->polarity);
            if(g_io_pwm_table[i].invert == 1){
                pwm_revert(g_io_pwm_table[i].channel);
            }
            else{
                pwm_n_revert(g_io_pwm_table[i].channel);
            }
        }   
    }
    else{
        if(TUYA_PWM_POSITIVE == cfg->polarity){
            tkl_pwm_debug("polarity ch id:%d,value:%d \r\n",ch_id,cfg->polarity);
            if(g_io_pwm_table[i].invert == 1){
                pwm_n_revert(g_io_pwm_table[i].channel);
            }
            else{
                pwm_revert(g_io_pwm_table[i].channel);
            }
        }
        else{
            tkl_pwm_debug("polarity ch id:%d,value:%d \r\n",ch_id,cfg->polarity);
            if(g_io_pwm_table[i].invert == 1){
                pwm_n_revert(g_io_pwm_table[i].channel);
            }
            else{
                pwm_revert(g_io_pwm_table[i].channel);
            }
        }    
    }

    drv_pwm_cfg(g_io_pwm_table[i].channel,0,pwm_max_tick); 

    gpio_set_func(g_io_pwm_table[i].gpio, g_io_pwm_table[i].func);

    if(g_firt_in[ch_id] == 0){
        g_firt_in[ch_id] = 1; 
    }

    return OPRT_OK;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_pwm_mapping_to_gpio(TUYA_PWM_NUM_E ch_id, UINT32_T gpio)
{
    UINT8_T index = 0;

    if(ch_id >= TKL_PWM_MAX_CHANNEL) {
        return OPRT_COM_ERROR;
    }

    UINT32_T gpio_tlsr = tkl_gpio_to_tlsr_gpio(gpio);
    index = get_io_pwm_map_index(gpio_tlsr);
    tkl_pwm_debug("find index:%d  gpio =%d\r\n",index, gpio);
    if(index != 0xFF) {
        g_pwm_table_index[ch_id] = index;
        tkl_pwm_debug("find ch id:%d\r\n",index);
        return OPRT_OK;
    }

    return OPRT_COM_ERROR;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_pwm_deinit(TUYA_PWM_NUM_E ch_id)
{
    if(ch_id >= TKL_PWM_MAX_CHANNEL) {
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}


/**
 * @description: set pwm value
 * @param {ch_id} pwm channel
 * @param {value} value want to be set
 * @return: 1 success,0 failed
 */
TUYA_WEAK_ATTRIBUTE OPERATE_RET set_pwm_value(UINT8_T ch_id, UINT32_T value)
{
    UINT8_T i = 0;
    UINT32_T pwm_max_tick = 0;

    if(ch_id >= TKL_PWM_MAX_CHANNEL) {
        return OPRT_COM_ERROR;
    }

    i = g_pwm_table_index[ch_id];

    if((i==0xFF) || (!g_pwm_precision[ch_id])){
        return OPRT_COM_ERROR;
    }

    drv_pwm_init();
    tkl_pwm_debug("set_pwm_value:%d,%d,%d\n",ch_id, value, g_pwm_precision[ch_id]);
    pwm_set_mode(g_io_pwm_table[i].channel, PWM_NORMAL_MODE);
	pwm_set_phase(g_io_pwm_table[i].channel, 0);   //no phase at pwm beginning
    pwm_max_tick = get_pwm_max_tick(tkl_pwm_cfg[ch_id].frequency);
	pwm_set_cycle_and_duty(g_io_pwm_table[i].channel, pwm_max_tick, value);

    config_pwm_polarity(ch_id);

    pwm_start(g_io_pwm_table[i].channel);

#if END_DEVICE
    hardware_set_wakeup(DEV_PWM_WAKE_UP_BITMAPS);
#endif

    return OPRT_OK;
}

/**
 * @description: set pwm duty cycle
 * @param {ch_id} pwm channel
 * @param {duty} duty cycle
 * @param {precision} Duty cycle precision
 * @return: 1 success,0 failed
 */
TUYA_WEAK_ATTRIBUTE OPERATE_RET set_pwm_duty(UINT8_T ch_id, UINT16_T duty, UINT16_T precision)
{
    UINT32_T value = 0;
    UINT8_T ret = 0;
    UINT8_T i = 0;
    UINT32_T pwm_max_tick = 0;

    tkl_pwm_debug("set_pwm_duty:%d,%d,%d\n",ch_id, duty, precision);
    if((ch_id >= TKL_PWM_MAX_CHANNEL) || (!precision)){
        return OPRT_COM_ERROR;
    }

    i = g_pwm_table_index[ch_id];

    if(i==0xFF){
        return OPRT_COM_ERROR;
    }
    
    tkl_pwm_cfg[ch_id].duty = duty;
    g_pwm_precision[ch_id] = precision;
    pwm_max_tick = get_pwm_max_tick(tkl_pwm_cfg[ch_id].frequency);
    value = (duty * pwm_max_tick) / precision;
    ret = set_pwm_value(ch_id, value);
    return ret;
}


TUYA_WEAK_ATTRIBUTE OPERATE_RET hal_pwm_start_sync(void)
{
    UINT32_T value = 0;
    UINT8_T index=0;
    UINT8_T channel_value=0;
    UINT8_T total_channel=0;
    UINT32_T pwm_max_tick = 0;

    for(index=0; index<TKL_PWM_MAX_CHANNEL ;index++)
    {
        channel_value = g_pwm_table_index[index];
        if(0xFF != channel_value){
            pwm_max_tick = get_pwm_max_tick(tkl_pwm_cfg[index].frequency);
            value = (tkl_pwm_cfg[index].duty * pwm_max_tick) / g_pwm_precision[index];
            drv_pwm_init();
            tkl_pwm_debug("set_pwm_value:%d,%d,%d,duty:%d\n",index, value, g_pwm_precision[index],tkl_pwm_cfg[index].duty);
            pwm_set_mode(g_io_pwm_table[channel_value].channel, PWM_NORMAL_MODE);
            pwm_set_phase(g_io_pwm_table[channel_value].channel, 0);   //no phase at pwm beginning
            pwm_set_cycle_and_duty(g_io_pwm_table[channel_value].channel, pwm_max_tick, value);
            total_channel |= BIT(g_io_pwm_table[channel_value].channel);

            config_pwm_polarity(index);
        }
    }
    tkl_pwm_debug("hal_pwm_start_sync total_channel:%x \r\n",total_channel);
    if(total_channel){
        pwm_start(total_channel | BIT(7));   
#if END_DEVICE
        hardware_set_wakeup(DEV_PWM_WAKE_UP_BITMAPS);
#endif   
        return  OPRT_OK;
    }
    return  OPRT_COM_ERROR;
}


TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_pwm_start(TUYA_PWM_NUM_E ch_id)
{
    if(ch_id < TKL_PWM_MAX_CHANNEL) {
        return set_pwm_duty(ch_id, tkl_pwm_cfg[ch_id].duty, g_pwm_precision[ch_id]);
    }
    else if(0xFF == ch_id){
        hal_pwm_start_sync();
    }

    return OPRT_OK;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_pwm_stop(TUYA_PWM_NUM_E ch_id)
{
    UINT8_T i=0;
    UINT8_T index=0;
    UINT8_T channel_value=0;
    UINT8_T total_channel=0;

    if((ch_id >= TKL_PWM_MAX_CHANNEL) && (ch_id != 0xFF)){
        return OPRT_COM_ERROR;
    }
    if(ch_id != 0xFF){
        i = g_pwm_table_index[ch_id];

        if(i==0xFF){
            return OPRT_COM_ERROR;
        }

        pwm_stop(g_io_pwm_table[i].channel);
    }
    else{
        for(index=0; index<TKL_PWM_MAX_CHANNEL ;index++){
            channel_value = g_pwm_table_index[index];
            if(0xFF != channel_value){
                total_channel |= BIT(g_io_pwm_table[channel_value].channel);
            }
        }
        tkl_pwm_debug("tkl_pwm_stop total_channel:%x \r\n",total_channel);
        if(total_channel){
           pwm_stop(total_channel | BIT(7)); 
        }
    }
#if END_DEVICE
    hardware_clear_wakeup(DEV_PWM_WAKE_UP_BITMAPS);
#endif
    return OPRT_OK;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_pwm_duty_set(TUYA_PWM_NUM_E ch_id, UINT32_T duty)
{
    UINT8_T i=0;

    if(ch_id >= TKL_PWM_MAX_CHANNEL) {
        return OPRT_COM_ERROR;
    }

    if(!g_pwm_precision[ch_id]){
        return OPRT_COM_ERROR;
    }

    i = g_pwm_table_index[ch_id];

    if(i==0xFF){
        return OPRT_COM_ERROR;
    }

    tkl_pwm_cfg[ch_id].duty = duty;

    set_pwm_duty(ch_id, tkl_pwm_cfg[ch_id].duty, g_pwm_precision[ch_id]);


    return OPRT_OK;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_pwm_frequency_set(TUYA_PWM_NUM_E ch_id, UINT32_T frequency)
{
    UINT8_T i=0;

    if(ch_id >= TKL_PWM_MAX_CHANNEL) {
        return OPRT_COM_ERROR;
    }

    if((!frequency) || (!g_pwm_precision[ch_id])){
        return OPRT_COM_ERROR;
    }

    i = g_pwm_table_index[ch_id];

    if(i==0xFF){
        return OPRT_COM_ERROR;
    }

    tkl_pwm_cfg[ch_id].frequency = frequency;

    UINT32_T pwm_max_tick  = get_pwm_max_tick(tkl_pwm_cfg[ch_id].frequency);
    UINT32_T pwm_duty = (UINT32_T)((pwm_max_tick * tkl_pwm_cfg[ch_id].duty) / g_pwm_precision[ch_id]);
    pwm_set_cycle_and_duty(g_io_pwm_table[i].channel, pwm_max_tick, pwm_duty);
    return OPRT_OK;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_pwm_polarity_set(TUYA_PWM_NUM_E ch_id, TUYA_PWM_POLARITY_E polarity)
{
    UINT8_T i=0;

    if(ch_id >= TKL_PWM_MAX_CHANNEL) {
        return OPRT_COM_ERROR;
    }

    i = g_pwm_table_index[ch_id];

    if(i==0xFF){
        return OPRT_COM_ERROR;
    }

    tkl_pwm_cfg[ch_id].polarity = polarity;
    
    if(g_pwm_idle_level[ch_id]){
        if(TUYA_PWM_POSITIVE == polarity){
            tkl_pwm_debug("polarity ch id:%d,value:%d \r\n",ch_id,polarity);
            if(g_io_pwm_table[i].invert == 1){
                pwm_revert(g_io_pwm_table[i].channel);
            }
            else{
                pwm_n_revert(g_io_pwm_table[i].channel);
            }
        }
        else{
            tkl_pwm_debug("polarity ch id:%d,value:%d \r\n",ch_id,polarity);
            if(g_io_pwm_table[i].invert == 1){
                pwm_revert(g_io_pwm_table[i].channel);
            }
            else{
                pwm_n_revert(g_io_pwm_table[i].channel);
            }
        }   
    }
    else{
        if(TUYA_PWM_POSITIVE == polarity){
            tkl_pwm_debug("polarity ch id:%d,value:%d \r\n",ch_id,polarity);
            if(g_io_pwm_table[i].invert == 1){
                pwm_n_revert(g_io_pwm_table[i].channel);
            }
            else{
                pwm_revert(g_io_pwm_table[i].channel);
            }
        }
        else{
            tkl_pwm_debug("polarity ch id:%d,value:%d \r\n",ch_id,polarity);
            if(g_io_pwm_table[i].invert == 1){
                pwm_n_revert(g_io_pwm_table[i].channel);
            }
            else{
                pwm_revert(g_io_pwm_table[i].channel);
            }
        }    
    }

    return OPRT_OK;
}


TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_pwm_info_set(TUYA_PWM_NUM_E ch_id, CONST TUYA_PWM_BASE_CFG_T *info)
{
    BOOL_T change_flag = FALSE;
    if((ch_id >= TKL_PWM_MAX_CHANNEL) || (!info)){
        return OPRT_COM_ERROR;
    }

    if(!g_pwm_precision[ch_id]){
        return OPRT_COM_ERROR;  
    }

    if(0 == tkl_system_memcmp(info, &tkl_pwm_cfg[ch_id], sizeof(TUYA_PWM_BASE_CFG_T))){
        return OPRT_OK;
    }
    if(info->frequency!=tkl_pwm_cfg[ch_id].frequency){
        tkl_pwm_frequency_set(ch_id, info->frequency);
        change_flag = TRUE;      
    }
    if(info->polarity!=tkl_pwm_cfg[ch_id].polarity){
        tkl_pwm_polarity_set(ch_id, info->polarity);
        change_flag = TRUE; 
    }
    if((info->duty!=tkl_pwm_cfg[ch_id].duty)||change_flag){
        tkl_pwm_duty_set(ch_id, info->duty);        
    }
    return OPRT_OK;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_pwm_info_get(TUYA_PWM_NUM_E ch_id, TUYA_PWM_BASE_CFG_T *info)
{
    if((ch_id >= TKL_PWM_MAX_CHANNEL) || (!info)){
        return OPRT_COM_ERROR;
    }

    if(!g_pwm_precision[ch_id]){
        return OPRT_COM_ERROR;  
    }

    tkl_system_memcpy(info, &tkl_pwm_cfg[ch_id], sizeof(TUYA_PWM_BASE_CFG_T));
    return OPRT_OK;
}
TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_pwm_idle_level_set(TUYA_PWM_NUM_E ch_id, UINT8_T idle_level)
{
    if(ch_id >= TKL_PWM_MAX_CHANNEL){
        return OPRT_COM_ERROR;
    }

    g_pwm_idle_level[ch_id] = idle_level;

    return OPRT_OK;
}

TUYA_WEAK_ATTRIBUTE OPERATE_RET tkl_pwm_precision_set(TUYA_PWM_NUM_E ch_id, UINT32_T precision)
{
    g_pwm_precision[ch_id] = precision;
    return OPRT_OK;
}


TUYA_WEAK_ATTRIBUTE void tkl_pwm_func_recovery(void)
{
#if END_DEVICE    
    UINT8_T i = 0;

    for(UINT8_T index=0;index<(sizeof(g_firt_in)/g_firt_in[0]);index++){
        if(g_firt_in[index]){
            i = g_pwm_table_index[index];
            if(i<sizeof(g_io_pwm_table)/sizeof(g_io_pwm_table[0])){
                gpio_set_func(g_io_pwm_table[i].gpio, g_io_pwm_table[i].func);
            }
        }
    }
#endif
}
